package othello;

import othello.model.* ;
import othello.view.*;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.* ;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
/**
 * <p>Title: Othello</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

public class ClientMain implements GameConstants {
    public static void main(String[] args)  {

		RemoteGameModelInterface proxy =null ;
		String name = null ;
		if( args.length > 0 ) name = args[0] ;
		int myColour = EMPTY ;
    	while(proxy == null) {
    		final int port = 1491 ;
    		final String defaultName = "gameModel" ;
    		String ip = "localhost";
			String suggestedURL = "rmi://" +ip+ ":" +port+ "/" +defaultName;
			
    		name = JOptionPane.showInputDialog("Enter the URL for the server", suggestedURL) ;
    		if( name == null ) System.exit(1) ;
    		try {
    			System.out.println("About to lookup "+name) ;
    			proxy = (RemoteGameModelInterface) Naming.lookup( name ) ; }
    		catch( java.lang.ClassCastException e ) {
    			JOptionPane.showMessageDialog(null, "Could not hook up with server.\nType mismatch") ;
    			e.printStackTrace( System.err ); }
    		catch( java.net.MalformedURLException  e) {
    			JOptionPane.showMessageDialog(null, "Could not hook up with server.\nURL not valid.") ;
    			e.printStackTrace( System.err ); }
    		catch( NotBoundException e) {
    			JOptionPane.showMessageDialog(null, "Could not hook up with server.\nName not bound on server.") ;
    			e.printStackTrace( System.err );  }
    		catch( RemoteException e) {
    			JOptionPane.showMessageDialog(null, "Could not hook up with server.\nNetwork problem.") ;
    			e.printStackTrace( System.err ); } 
    		catch( Throwable e ) {
    			JOptionPane.showMessageDialog(null, "Could not hook up with server.") ;
    			e.printStackTrace( System.err ); }	}
		
		System.out.println("Have a proxy") ;
		
        Animator animator ;
        try {
            animator = new Animator( ) ;
            animator.setGameModel(proxy); }
        catch( RemoteException e ) {
            animator = null ;
            System.err.println("Could not create animator");
            e.printStackTrace( System.err );
            System.exit(1); }
		
		buildGUI(proxy, animator ) ; 
		
		try {
			int colour = proxy.getColour();
			animator.setColour(colour); 
		} catch (RemoteException e) {
            System.err.println("Could not get colour");
			e.printStackTrace();
		}
    }
    
    private static void buildGUI(final RemoteGameModelInterface proxy,
    		final Animator animator) {
    	SwingUtilities.invokeLater(new Runnable(){

    		@Override
    		public void run() {  
    			ViewStrategy perspective = new BirdsEyeViewStrategy() ;
    			BoardView view0 = new BoardView( animator, perspective ) ;
    			JFrame frame0 = new JFrame() ;
    			frame0.setDefaultCloseOperation( WindowConstants.EXIT_ON_CLOSE ) ;
    			frame0.add( view0 ) ;
    			frame0.setSize(350, 350);
    			frame0.setVisible( true );

    			//( myColour == WHITE ) frame0.setBounds( 0, 350, 350, 350 ) ;

    		}});
    }
}
