package othello.view;

import java.awt.* ;
import java.awt.geom.Point2D;

import othello.model.Pair;
/** A simple perspective transform.
 * <p>Title: Othello</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

public class PerspectiveViewStrategy implements ViewStrategy {

    Transform3D xform ;
    
    public PerspectiveViewStrategy() {
    	
/** @todo Add a VUP vector */
        double[] camera = new double[]{-10.0, -5.0, 10.0, 1.0} ;
        double[] lookAt = new double[]{5.0, 5.0, 0.0, 1.0} ;
        Transform3D xlateToOrigin = Transform3D.createTranslation(
                    -lookAt[0], -lookAt[1], -lookAt[2]) ;
                    
        xform = xlateToOrigin ;
//        double[] camera_1 = xlateToOrigin.apply( camera ) ;
//        // Rotate around the Z axis till camera is in the x=0 plane
//        System.err.println( "camera_1 = ("+camera_1[0]+", "+camera_1[1]+", "+camera_1[2]+")");
//        double angle = Math.atan2( camera_1[0], camera_1[1] ) ;
//        System.err.println( "rotation around Z = "+angle);
//        Transform3D rotateZ = Transform3D.createRotationZ( angle ) ;
//        xform = Transform3D.compose( rotateZ, xform ) ;
//        
//        double[] camera_2 = xform.apply( camera ) ;
//        // Rotate around X axis until the camera is on the z axis
//        System.err.println( "camera_2 = ("+camera_2[0]+", "+camera_2[1]+", "+camera_2[2]+")");
//        angle = Math.atan2( camera_2[1], camera_2[2] ) ;
//        System.err.println( "rotation around X= "+ angle);
//        Transform3D rotateX = Transform3D.createRotationX( angle ) ;
//        xform = Transform3D.compose( rotateX, xform ) ;
//        
//        double[] camera_3 = xform.apply( camera ) ;
//        System.err.println( "camera_3 = ("+camera_3[0]+", "+camera_3[1]+", "+camera_3[2]+")");
//        Transform3D xlateUp = Transform3D.createTranslation(0, 0, camera_3[2]) ;
//        xform = Transform3D.compose( xlateUp, xform ) ;
        
        Transform3D perspective = Transform3D.createPerspective( 20, 1.0, 1.0 ) ;
        xform = Transform3D.compose( perspective, xform ) ;
    }
    
    public Point2D worldToView( double width, double height, double row, double col ) {
        // (row, col) is a position in the world.
        // We need to (a) map it to coordinates relative to the camera
        // and (b) do a perspective projection onto the view plane.
        // Both these transformations are done by applying xform.
            double[] p = new double[]{ row, col, 0, 1.0 } ;
            p = xform.apply( p ) ;
            
            col = -p[0] ;
            row = p[1] ;
        
        // Now (row, col) is a position on the view plane, where
        // the origin of the view plane is on the camera to lookat point.
        // We take the view area to be a 20.0 by 20.0 square.
        // It remains to map positions on the view plane to pixel coordinates (x,y).
            double vertStretch = height / 20.0 ;
            double horStretch = width / 20.0 ;
            horStretch = vertStretch = Math.min( horStretch, vertStretch ) ;
            double y = (10.0+row)*vertStretch ;
            double x = (10.0+col)*horStretch ;
        return new Point2D.Double( x, y ) ; }

    
    /**
     * TODO This is not yet correct. Do not use!
     */
    public Pair viewToWorld( double width, double height, double x, double y ) {
    	// See worldToView. Here we run the process backwards.
    	double vertStretch = height / 20.0 ;
        double horStretch = width / 20.0 ;
        horStretch = vertStretch = Math.min( horStretch, vertStretch ) ;
        double row = y/vertStretch - 10.0 ;
        double col = x/horStretch - 10.0 ;
        double[] p = new double[]{ -col, row, 1.0, 1.0 } ; // Does z matter?
        p = xform.applyInverse( p ) ;
        return new Pair( (int)p[0], (int)p[1] ) ; }
}

class Transform3D {
    double[][] a = new double [4][4] ;
    double[][] b = new double [4][4] ;
    
    private Transform3D() {}
    
    static Transform3D createScale(double x, double y, double z) {
        Transform3D t = new Transform3D() ;
        t.a[0][0] = x ;
        t.a[1][1] = y ;
        t.a[2][2] = z ;
        t.a[3][3] = 1.0 ;
        t.b[0][0] = 1/x ;
        t.b[1][1] = 1/y ;
        t.b[2][2] = 1/z ;
        t.b[3][3] = 1 ;
        return t ; }
    
    static Transform3D createTranslation(double x, double y, double z) {
        Transform3D t = new Transform3D() ;
        t.a[0][3] = x ;
        t.a[1][3] = y ;
        t.a[2][3] = z ;
        t.a[0][0] = t.a[1][1] = t.a[2][2] = t.a[3][3] = 1.0 ;
        
        t.b[0][3] = -x ;
        t.b[1][3] = -y ;
        t.b[2][3] = -z ;
        t.b[0][0] = t.b[1][1] = t.b[2][2] = t.b[3][3] = 1.0 ;
        return t ; }
    
    static Transform3D createRotationX(double theta) {
        double s = Math.sin(theta) ;
        double c = Math.cos(theta) ;
        Transform3D t = new Transform3D() ;
        t.a[0][0] = t.a[3][3] = 1.0 ;
        t.a[1][1] = t.a[2][2] = c ;
        t.a[1][2] = -s ;
        t.a[2][1] = s ;

        t.b[0][0] = t.b[3][3] = 1.0 ;
        t.b[1][1] = t.b[2][2] = c ;
        t.b[1][2] = s ;
        t.b[2][1] = -s ;
        return t ; }
    
    static Transform3D createRotationY(double theta) {
        double s = Math.sin(theta) ;
        double c = Math.cos(theta) ;
        Transform3D t = new Transform3D() ;
        t.a[1][1] = t.a[3][3] = 1.0 ;
        t.a[0][0] = t.a[2][2] = c ;
        t.a[2][0] = -s ;
        t.a[0][2] = s ;

        t.b[1][1] = t.b[3][3] = 1.0 ;
        t.b[0][0] = t.b[2][2] = c ;
        t.b[2][0] = s ;
        t.b[0][2] = -s ;
        return t ; }
    
    static Transform3D createRotationZ(double theta) {
        double s = Math.sin(theta) ;
        double c = Math.cos(theta) ;
        Transform3D t = new Transform3D() ;
        t.a[2][2] = t.a[3][3] = 1.0 ;
        t.a[0][0] = t.a[1][1] = c ;
        t.a[0][1] = -s ;
        t.a[1][0] = s ;

        t.b[2][2] = t.b[3][3] = 1.0 ;
        t.b[0][0] = t.b[1][1] = c ;
        t.b[0][1] = s ;
        t.b[1][0] = -s ;
        return t ; }
        
    
    static Transform3D createPerspective(double N, double a, double b) {
        Transform3D t = new Transform3D() ;
        t.a[0][0] = N ;
        t.a[1][1] = N ;
        t.a[2][2] = a ;
        t.a[2][3] = b ;
        t.a[3][2] = -1.0 ;
        

        t.b[0][0] = 1/N ;
        t.b[1][1] = 1/N ;
        t.b[2][3] = -1 ;
        t.b[3][2] = 1/b ;
        t.b[3][3] = a/b ;
        return t ; }
    
    static Transform3D compose( Transform3D t, Transform3D u ) {
        
        Transform3D v = new Transform3D() ;
        for( int i=0 ; i < 4 ; ++i ) {
            for( int j=0 ; j < 4 ; ++j ) {
                double s = 0.0 ;
                for( int k = 0 ; k < 4 ; ++k ) {
                    s += t.a[i][k] * u.a[k][j] ; }
                v.a[i][j] = s ; } }
        
        for( int i=0 ; i < 4 ; ++i ) {
            for( int j=0 ; j < 4 ; ++j ) {
                double s = 0.0 ;
                for( int k = 0 ; k < 4 ; ++k ) {
                    s += u.b[i][k] * t.b[k][j] ; }
                v.b[i][j] = s ; } }
        
        return v ; }
    
    double[]  apply( double[] p ) {
        double[] r = new double[4] ;
        for( int i=0 ; i < 4 ; ++i ) {
            double s = 0.0 ;
            for( int k = 0 ; k < 4 ; ++k ) {
                s += a[i][k] * p[k] ; }
            r[i] = s ; }
        // Normalize.
        for( int i = 0 ; i < 4 ; ++i ) {
            r[i] = r[i] / r[3] ; }
        return r ; }
    
    double[]  applyInverse( double[] p ) {
        double[] r = new double[4] ;
        for( int i=0 ; i < 4 ; ++i ) {
            double s = 0.0 ;
            for( int k = 0 ; k < 4 ; ++k ) {
                s += b[i][k] * p[k] ; }
            r[i] = s ; }
        // Normalize.
        for( int i = 0 ; i < 4 ; ++i ) {
            r[i] = r[i] / r[3] ; }
        return r ; }
}